from dataclasses import dataclass, asdict, field
from cooptools.config import JsonConfigHandler
from typing import List, Iterable, Dict, Any
import datetime
from cooptools.typevalidation import datestamp_tryParse
import json
import uuid
import cooptools.date_utils as du
from cooptools.coopEnum import CoopEnum
from cooptools.protocols import UniqueIdentifier
from cooptools import materialHandling as mh

UNDEFINED ='UNDEFINED'

class TaskLinkType(CoopEnum):
    PREDECESSOR = 'predecessor'
    PARENT = 'parent'

@dataclass(frozen=True, slots=True)
class TaskLink:
    task_link_type: TaskLinkType
    linked_task_id: str

@dataclass(frozen=True, slots=True)
class ExecutionCriteria:
    criteria_payload: Any = field(default_factory=dict)
    required_agent_id_options: Iterable[str] = field(default_factory=list)
    required_agent_type_options: Iterable[str] = field(default_factory=list)

@dataclass(frozen=True, slots=True)
class HeirarchyCriteria:
    task_group_id: UniqueIdentifier = field(default=UNDEFINED)
    task_links: Iterable[TaskLink] = field(default_factory=list)

@dataclass(frozen=True, slots=True)
class TaskMeta:
    id: UniqueIdentifier
    task_type: str
    creating_user_id: UniqueIdentifier = field(default=UNDEFINED)
    created_date: datetime.datetime = field(default_factory=du.now)
    due_date: datetime.datetime = None
    expiration_date: datetime.datetime = None
    priority: int = 999

    def __post_init__(self):
        object.__setattr__(self, f'{self.due_date=}'.split('=')[0].replace('self.', ''), datestamp_tryParse(self.due_date))
        object.__setattr__(self, f'{self.created_date=}'.split('=')[0].replace('self.', ''), datestamp_tryParse(self.created_date))
        object.__setattr__(self, f'{self.expiration_date=}'.split('=')[0].replace('self.', ''), datestamp_tryParse(self.expiration_date))
        if type(self.priority) == str:
            object.__setattr__(self, f'{self.priority=}'.split('=')[0].replace('self.', ''), int(self.priority))

    def toJson(self):
        return json.dumps(asdict(self), default=str, indent=4)

@dataclass(frozen=True, slots=True)
class Task:
    meta: TaskMeta
    execution_criteria: ExecutionCriteria
    heirarchy_criteria: HeirarchyCriteria = field(default_factory=HeirarchyCriteria)

    def __post_init__(self):
        if type(self.meta) == dict:
            object.__setattr__(self, f'{self.meta=}'.replace('self.', '').split('=')[0], TaskMeta(**self.meta))

@dataclass(frozen=True, slots=True)
class MissionCriteriaPayload:
    task_names: Iterable[str]

@dataclass(frozen=True, slots=True)
class MoveCriteriaPayload:
    waypoints: Iterable[str]

@dataclass(frozen=True, slots=True)
class LoadTransferCriteriaPayload:
    load_meta: mh.Load




if __name__ == '__main__':
    from pprint import pprint

    def test_base_args():
        task = Task(
            meta=TaskMeta(
                id='1',
                task_type="user type",
            ),
            execution_criteria=ExecutionCriteria(
                criteria_payload=MissionCriteriaPayload(
                    task_names=['2', '3', '4'],
                ),
            )
        )
        pprint(task)

    test_base_args()